# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import abc
from collections import namedtuple
from skbio import TreeNode
import pandas as pd
import numpy


class Dendrogram(TreeNode):
    """ Stores data to be plotted as a dendrogram.

    A `Dendrogram` object is represents a tree in addition to the
    key information required to create a tree layout prior to
    visualization.  No layouts are specified within this class,
    since this serves as a super class for different tree layouts.

    Parameters
    ----------
    use_lengths: bool
        Specifies if the branch lengths should be included in the
        resulting visualization (default True).

    Attributes
    ----------
    length
    leafcount
    height
    depth

    Notes
    -----
    `length` refers to the branch length of a node to its parent.
    `leafcount` is the number of tips within a subtree. `height` refers
    to the longest path from root to the deepst leaf in that subtree.
    `depth` is the number of nodes found in the longest path.

    """
    def __init__(self, use_lengths=True, **kwargs):
        """ Constructs a Dendrogram object for visualization.
        """
        super().__init__(**kwargs)

    def _cache_ntips(self):
        """ Counts the number of leaves under each subtree."""
        for n in self.postorder():
            if n.is_tip():
                n.leafcount = 1
            else:
                n.leafcount = sum(c.leafcount for c in n.children)

    def update_geometry(self, use_lengths, depth=None):
        """Calculate tree node attributes such as height and depth.

        Parameters
        ----------
        use_lengths: bool
           Specify if the branch length should be incorporated into
           the geometry calculations for visualization.
        depth: int
           The number of nodes in the longest path from root to leaf.

        This is agnostic to scale and orientation.
        """
        if self.length is None or not use_lengths:
            if depth is None:
                self.length = 0
            else:
                self.length = 1

        self.depth = (depth or 0) + self.length

        children = self.children
        if children:
            for c in children:
                c.update_geometry(use_lengths, self.depth)
            self.height = max([c.height for c in children]) + self.length
            self.leafcount = sum([c.leafcount for c in children])

        else:
            self.height = self.length
            self.leafcount = self.edgecount = 1

    def coords(self, height, width):
        """ Returns coordinates of nodes to be rendered in plot.

        Parameters
        ----------
        height : int
            The height of the canvas.
        width : int
            The width of the canvas.
        Returns
        -------
        pd.DataFrame
            index : str
                Name of node.
            x : float
                x-coordinate of node.
            y : float
                y-coordinate of node.
            child(i) : str
                Name of ith child node in that specific node.
                in the tree.
            is_tip : str
                Specifies if the node is a tip in the treee.
        """
        self.rescale(width, height)
        result = {}
        for node in self.postorder():
            children = {'child%d' % i: n.name
                        for i, n in enumerate(node.children)}
            coords = {'x': node.x2, 'y': node.y2}
            is_tip = {'is_tip': node.is_tip()}
            result[node.name] = {**coords, **children, **is_tip}
        result = pd.DataFrame(result).T

        # reorder so that x and y are first
        cols = ['x', 'y'] + sorted(list(set(result.columns) - set(['x', 'y'])))
        return result.loc[:, cols]

    @abc.abstractmethod
    def rescale(self, width, height):
        pass


class UnrootedDendrogram(Dendrogram):
    """ Stores data to be plotted as an unrooted dendrogram.

    A `UnrootedDendrogram` object is represents a tree in addition to the
    key information required to create a radial tree layout prior to
    visualization.

    Parameters
    ----------
    use_lengths: bool
        Specifies if the branch lengths should be included in the
        resulting visualization (default True).

    Attributes
    ----------
    length
    leafcount
    height
    depth
    """
    def __init__(self, **kwargs):
        """ Constructs a UnrootedDendrogram object for visualization.

        Parameters
        ----------
        use_lengths: bool
            Specifies if the branch lengths should be included in the
            resulting visualization (default True).
        """
        super().__init__(**kwargs)

    @classmethod
    def from_tree(cls, tree, use_lengths=True):
        """ Creates an UnrootedDendrogram object from a skbio tree.

        Parameters
        ----------
        tree : skbio.TreeNode
            Input skbio tree

        Returns
        -------
        UnrootedDendrogram
        """
        for n in tree.postorder():
            n.__class__ = UnrootedDendrogram

        tree.update_geometry(use_lengths)
        return tree

    def rescale(self, width, height):
        """ Find best scaling factor for fitting the tree in the figure.

        This method will find the best orientation and scaling possible
        to fit the tree within the dimensions specified by width and height.

        Parameters
        ----------
        width : float
            width of the canvas
        height : float
            height of the canvas

        Returns
        -------
        best_scaling : float
            largest scaling factor in which the tree can fit in the canvas.

        Notes
        -----
        """
        angle = (2 * numpy.pi) / self.leafcount
        # this loop is a horrible brute force hack
        # there are better (but complex) ways to find
        # the best rotation of the tree to fit the display.
        best_scale = 0
        for i in range(60):
            direction = i / 60.0 * numpy.pi
            # TODO:
            # This function has a little bit of recursion.  This will
            # need to be refactored to remove the recursion.

            points = self.update_coordinates(1.0, 0, 0, direction, angle)
            xs, ys = zip(*points)
            # double check that the tree fits within the margins
            scale = min(float(width) / (max(xs) - min(xs)),
                        float(height) / (max(ys) - min(ys)))
            # TODO: This margin seems a bit arbituary.
            # will need to investigate.
            scale *= 0.95  # extra margin for labels
            if scale > best_scale:
                best_scale = scale
                mid_x = width / 2 - ((max(xs) + min(xs)) / 2) * scale
                mid_y = height / 2 - ((max(ys) + min(ys)) / 2) * scale
                best_args = (scale, mid_x, mid_y, direction, angle)

        self.update_coordinates(*best_args)
        return best_scale

    def update_coordinates(self, s, x1, y1, a, da):
        """ Update x, y coordinates of tree nodes in canvas.

        `update_coordinates` will recursively updating the
        plotting parameters for all of the nodes within the tree.
        This can be applied when the tree becomes modified (i.e. pruning
        or collapsing) and the resulting coordinates need to be modified
        to reflect the changes to the tree structure.

        Parameters
        ----------
        s : float
            scaling
        x1 : float
            x midpoint
        y1 : float
            y midpoint
        a : float
            angle (degrees)
        da : float
            angle resolution (degrees)

        Returns
        -------
        points : list of tuple
            2D coordinates of all of the nodes.

        Notes
        -----
        This function has a little bit of recursion.  This will
        need to be refactored to remove the recursion.
        """
        # Constant angle algorithm.  Should add maximum daylight step.
        x2 = x1 + self.length * s * numpy.sin(a)
        y2 = y1 + self.length * s * numpy.cos(a)
        (self.x1, self.y1, self.x2, self.y2, self.angle) = (x1, y1, x2, y2, a)
        # TODO: Add functionality that allows for collapsing of nodes
        a = a - self.leafcount * da / 2
        if self.is_tip():
            points = [(x2, y2)]
        else:
            points = []
            # TODO:
            # This function has a little bit of recursion.  This will
            # need to be refactored to remove the recursion.
            for child in self.children:
                # calculate the arc that covers the subtree.
                ca = child.leafcount * da
                points += child.update_coordinates(s, x2, y2, a+ca/2, da)
                a += ca
        return points


Dimensions = namedtuple('Dimensions', ['x', 'y', 'height'])


class RootedDendrogram(Dendrogram):
    """ Stores data to be plotted as an rooted dendrogram.

    A `RootedDendrogram` object is represents a tree in addition to the
    key information required to create a radial tree layout prior to
    visualization.

    Parameters
    ----------
    use_lengths: bool
        Specifies if the branch lengths should be included in the
        resulting visualization (default True).

    Attributes
    ----------
    length
    leafcount
    height
    depth
    """

    def width_required(self):
        return self.leafcount

    @abc.abstractmethod
    def xcoords(self, scale, x1):
        pass

    @abc.abstractmethod
    def ycoords(self, scale, y1):
        pass

    def rescale(self, width, height):
        """ Update x, y coordinates of tree nodes in canvas.

        Parameters
        ----------
        scale : Dimensions
            Scaled dimensions of the tree
        x1 : int
            X-coordinate of parent
        """
        xscale = width / self.height
        yscale = height / self.width_required()
        scale = Dimensions(xscale, yscale, self.height)

        # y coords done postorder, x preorder, y first.
        # so it has to be done in 2 passes.
        self.update_y_coordinates(scale)
        self.update_x_coordinates(scale)
        return xscale

    def update_y_coordinates(self, scale, y1=None):
        """The second pass through the tree.  Y coordinates only
        depend on the shape of the tree and yscale.

        Parameters
        ----------
        scale : Dimensions
            Scaled dimensions of the tree
        x1 : int
            X-coordinate of parent
        """
        if y1 is None:
            y1 = self.width_required() * scale.y
        child_y = y1
        for child in self.children:
            child.update_y_coordinates(scale, child_y)
            child_y -= child.width_required() * scale.y
        (self.y1, self.y2) = self.ycoords(scale, y1)

    def update_x_coordinates(self, scale, x1=0):
        """For non 'square' styles the x coordinates will depend
        (a bit) on the y coodinates, so they should be done first.
        Parameters
        ----------
        scale : Dimensions
            Scaled dimensions of the tree
        x1 : int
            X-coordinate of parent
        """
        (self.x1, self.x2) = self.xcoords(scale, x1)
        for child in self.children:
            child.update_x_coordinates(scale, self.x2)


class SquareDendrogram(RootedDendrogram):

    def ycoords(self, scale, y1):
        cys = [c.y1 for c in self.children]
        if cys:
            y2 = (cys[0]+cys[-1]) / 2.0
        else:
            y2 = y1 - 0.5 * scale.y
        return (y2, y2)

    def xcoords(self, scale, x1):
        if self.is_tip():
            return (x1, (scale.height-(self.height-self.length))*scale.x)
        else:
            # give some margins for internal nodes
            dx = scale.x * self.length * 0.95
            x2 = x1 + dx
            return (x1, x2)

    @classmethod
    def from_tree(cls, tree):
        """ Creates an SquareDendrogram object from a skbio tree.

        Parameters
        ----------
        tree : skbio.TreeNode
            Input skbio tree
        Returns
        -------
        SquareDendrogram
        """
        for n in tree.postorder(include_self=True):
            n.__class__ = SquareDendrogram
        tree.update_geometry(use_lengths=False)
        return tree



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import unittest
import os
import shutil

import numpy as np
import pandas as pd
import numpy.testing as npt

from skbio import TreeNode
from skbio.util import get_data_path

from gneiss.plot._regression_plot import ols_summary, lme_summary
from gneiss.regression import ols, mixedlm


class TestOLS_Summary(unittest.TestCase):

    def setUp(self):
        A = np.array  # aliasing for the sake of pep8
        self.table = pd.DataFrame({
            's1': A([1., 1.]),
            's2': A([1., 2.]),
            's3': A([1., 3.]),
            's4': A([1., 4.]),
            's5': A([1., 5.])},
            index=['Y2', 'Y1']).T
        self.tree = TreeNode.read(['(c, (b,a)Y2)Y1;'])
        self.metadata = pd.DataFrame({
            'lame': [1, 1, 1, 1, 1],
            'real': [1, 2, 3, 4, 5]
        }, index=['s1', 's2', 's3', 's4', 's5'])

        np.random.seed(0)
        n = 15
        a = np.array([1, 4.2, 5.3, -2.2, 8])
        x1 = np.linspace(.01, 0.1, n)
        x2 = np.logspace(0, 0.01, n)
        x3 = np.exp(np.linspace(0, 0.01, n))
        x4 = x1 ** 2
        self.x = pd.DataFrame({'x1': x1, 'x2': x2, 'x3': x3, 'x4': x4})
        y = (a[0] + a[1]*x1 + a[2]*x2 + a[3]*x3 + a[4]*x4 +
             np.random.normal(size=n))
        sy = np.vstack((-y/10, -y)).T
        self.y = pd.DataFrame(sy, columns=['y0', 'y1'])
        self.t2 = TreeNode.read([r"((a,b)y1,c)y0;"])

        self.results = "results"
        os.mkdir(self.results)

    def tearDown(self):
        shutil.rmtree(self.results)

    def test_visualization(self):
        res = ols(formula="x1 + x2 + x3 + x4",
                  table=self.y, metadata=self.x)
        res.fit()

        ols_summary(self.results, res, tree=self.t2)
        fp = os.path.join(self.results, 'pvalues.csv')
        self.assertTrue(os.path.exists(fp))
        fp = os.path.join(self.results, 'coefficients.csv')
        self.assertTrue(os.path.exists(fp))
        fp = os.path.join(self.results, 'predicted.csv')
        self.assertTrue(os.path.exists(fp))
        fp = os.path.join(self.results, 'residuals.csv')
        self.assertTrue(os.path.exists(fp))

        index_fp = os.path.join(self.results, 'index.html')
        self.assertTrue(os.path.exists(index_fp))

        with open(index_fp, 'r') as fh:
            html = fh.read()
            self.assertIn('<h1>Simplicial Linear Regression Summary</h1>',
                          html)
            self.assertIn('<th>Coefficients</th>\n', html)
            self.assertIn('<th>Predicted Balances</th>\n', html)
            self.assertIn('<th>Residuals</th>\n', html)


class TestLME_Summary(unittest.TestCase):

    def setUp(self):
        np.random.seed(6241)
        n = 1600
        exog = np.random.normal(size=(n, 2))
        groups = np.kron(np.arange(n // 16), np.ones(16))

        # Build up the random error vector
        errors = 0

        # The random effects
        exog_re = np.random.normal(size=(n, 2))
        slopes = np.random.normal(size=(n // 16, 2))
        slopes = np.kron(slopes, np.ones((16, 1))) * exog_re
        errors += slopes.sum(1)

        # First variance component
        errors += np.kron(2 * np.random.normal(size=n // 4), np.ones(4))

        # Second variance component
        errors += np.kron(2 * np.random.normal(size=n // 2), np.ones(2))

        # iid errors
        errors += np.random.normal(size=n)

        endog = exog.sum(1) + errors

        df = pd.DataFrame(index=range(n))
        df["Y1"] = endog + 2 * 2
        df["Y2"] = endog
        df["groups"] = groups
        df["x1"] = exog[:, 0]
        df["x2"] = exog[:, 1]

        self.tree = TreeNode.read(['(c, (b,a)Y2)Y1;'])
        self.table = df[["Y1", "Y2"]]
        self.metadata = df[['x1', 'x2', 'groups']]

        self.results = "results"
        if not os.path.exists(self.results):
            os.mkdir(self.results)

    def tearDown(self):
        shutil.rmtree(self.results)

    def test_visualization(self):
        model = mixedlm("x1 + x2", self.table, self.metadata,
                        groups="groups")
        model.fit()
        lme_summary(self.results, model, self.tree)
        pvals = pd.read_csv(os.path.join(self.results, 'pvalues.csv'),
                            index_col=0)
        coefs = pd.read_csv(os.path.join(self.results, 'coefficients.csv'),
                            index_col=0)
        pred = pd.read_csv(os.path.join(self.results, 'predicted.csv'),
                           index_col=0)
        resid = pd.read_csv(os.path.join(self.results, 'residuals.csv'),
                            index_col=0)

        exp_pvals = pd.DataFrame({
            'Intercept': {'Y1': 4.8268860492262526e-236,
                          'Y2': 0.099411090631406948},
            'groups RE': {'Y1': 4.4193804668281966e-05,
                          'Y2': 4.4193804668280984e-05},
            'x1': {'Y1': 3.9704936434633392e-35,
                   'Y2': 3.9704936434628853e-35},
            'x2': {'Y1': 3.56912071867573e-30,
                   'Y2': 3.56912071867573e-30}})
        npt.assert_allclose(pvals, exp_pvals, rtol=1e-5)

        exp_coefs = pd.DataFrame({
            'Intercept': {'Y1': 4.2115280233151946,
                          'Y2': 0.211528023315187},
            'groups RE': {'Y1': 0.093578639287859755,
                          'Y2': 0.093578639287860019},
            'x1': {'Y1': 1.0220072967452645,
                   'Y2': 1.0220072967452651},
            'x2': {'Y1': 0.92487193877761575,
                   'Y2': 0.92487193877761564}}
        )
        npt.assert_allclose(coefs, exp_coefs, rtol=1e-2, atol=1e-2)

        exp_resid = pd.read_csv(get_data_path('exp_resid.csv'), index_col=0)
        npt.assert_allclose(resid, exp_resid.T, rtol=1e-2, atol=1e-2)

        exp_pred = pd.read_csv(get_data_path('exp_pred.csv'), index_col=0)
        npt.assert_allclose(pred, exp_pred.T, rtol=1e-2, atol=1e-2)


if __name__ == "__main__":
    unittest.main()



from gneiss.plot import heatmap
from gneiss.plot._heatmap import _sort_table

import pandas as pd
import pandas.util.testing as pdt
from skbio import TreeNode, DistanceMatrix
from scipy.cluster.hierarchy import ward
from gneiss.plot._dendrogram import SquareDendrogram
from gneiss.util import block_diagonal
from gneiss.cluster import rank_linkage
import numpy as np
import numpy.testing.utils as npt
import unittest


class HeatmapTest(unittest.TestCase):
    def setUp(self):
        np.random.seed(0)
        self.table = pd.DataFrame(np.random.random((5, 5)),
                                  index=['0', '1', '2', '3', '4'],
                                  columns=['0', '1', '2', '3', '4'])

        num_otus = 5  # otus
        x = np.random.rand(num_otus)
        dm = DistanceMatrix.from_iterable(x, lambda x, y: np.abs(x-y))
        lm = ward(dm.condensed_form())
        t = TreeNode.from_linkage_matrix(lm, np.arange(len(x)).astype(np.str))
        self.t = SquareDendrogram.from_tree(t)
        self.md = pd.Series(['a', 'a', 'a', 'b', 'b'],
                            index=['0', '1', '2', '3', '4'])
        for i, n in enumerate(t.postorder()):
            if not n.is_tip():
                n.name = "y%d" % i
            n.length = np.random.rand()*3

        self.highlights = pd.DataFrame({'y8': ['#FF0000', '#00FF00'],
                                        'y6': ['#0000FF', '#F0000F']}).T

    def test_sort_table(self):
        table = pd.DataFrame(
            [[1, 1, 0, 0, 0],
             [0, 1, 1, 0, 0],
             [0, 0, 1, 1, 0],
             [0, 0, 0, 1, 1]],
            columns=['s1', 's2', 's3', 's4', 's5'],
            index=['o1', 'o2', 'o3', 'o4'])
        mdvar = pd.Series(['a', 'b', 'a', 'b', 'a'],
                          index=['s1', 's2', 's3', 's4', 's5'])
        res_table, res_mdvar = _sort_table(table, mdvar)
        pdt.assert_index_equal(pd.Index(['s1', 's3', 's5', 's2', 's4']),
                               res_mdvar.index)
        pdt.assert_index_equal(pd.Index(['s1', 's3', 's5', 's2', 's4']),
                               res_table.columns)

    def test_basic(self):
        fig = heatmap(self.table, self.t, self.md,
                      figsize=(5, self.table.shape[0]))

        # Test to see if the lineages of the tree are ok
        lines = list(fig.get_axes()[0].get_lines())

        exp_coords = np.array([[14.25, 0.5],
                               [14.25, 1.],
                               [14.25, 1.],
                               [20., 1.],
                               [9.5, 1.25],
                               [9.5, 2.],
                               [9.5, 2.],
                               [20., 2.],
                               [4.75, 2.125],
                               [4.75, 3.],
                               [4.75, 3.],
                               [20., 3.],
                               [0., 3.0625],
                               [0., 4.],
                               [0., 4.],
                               [20., 4.],
                               [14.25, 0.5],
                               [14.25, 0.],
                               [14.25, 0.],
                               [20., 0.],
                               [9.5, 1.25],
                               [9.5, 0.5],
                               [9.5, 0.5],
                               [14.25, 0.5],
                               [4.75, 2.125],
                               [4.75, 1.25],
                               [4.75, 1.25],
                               [9.5, 1.25],
                               [0., 3.0625],
                               [0., 2.125],
                               [0., 2.125],
                               [4.75, 2.125]])

        res = np.vstack([i._xy for i in lines])

        npt.assert_allclose(exp_coords, res)

        # Make sure that the metadata labels are set properly
        res = str(fig.get_axes()[1].get_xticklabels(minor=True)[0])
        self.assertEqual(res, "Text(0,0,'a')")

        res = str(fig.get_axes()[1].get_xticklabels(minor=True)[1])
        self.assertEqual(res, "Text(0,0,'b')")

        res = str(fig.get_axes()[1].get_xlabel())
        self.assertEqual(res, "None")

    def test_basic_line_width(self):
        fig = heatmap(self.table, self.t, self.md,
                      figsize=(5, self.table.shape[0]), linewidth=1)

        # Test to see if the lineages of the tree are ok
        lines = list(fig.get_axes()[1].get_lines())
        widths = [l.get_lw() for l in lines]
        np.allclose(widths, [1.0] * len(widths))

    def test_highlights(self):

        table = pd.DataFrame(block_diagonal(ncols=5, nrows=5, nblocks=2),
                             index=['0', '1', '2', '3', '4'],
                             columns=['0', '1', '2', '3', '4'])
        t = rank_linkage(pd.Series([1, 2, 3, 4, 5],
                                   index=['0', '1', '2', '3', '4']))
        t = SquareDendrogram.from_tree(t)
        md = pd.Series(['a', 'a', 'a', 'b', 'b'],
                       index=['0', '1', '2', '3', '4'])
        for i, n in enumerate(t.postorder()):
            if not n.is_tip():
                n.name = "y%d" % i
            n.length = np.random.rand()*3

        highlights = pd.DataFrame({'y8': ['#FF0000', '#00FF00'],
                                   'y7': ['#0000FF', '#F0000F']}).T

        fig = heatmap(table, t, md, highlights)

        # Test to see if the lineages of the tree are ok
        lines = list(fig.get_axes()[0].get_lines())

        pts = self.t.coords(width=20, height=self.table.shape[0])
        pts['y'] = pts['y'] - 0.5  # account for offset
        pts['x'] = pts['x'].astype(np.float)
        pts['y'] = pts['y'].astype(np.float)

        exp_coords = np.array([[6.33333333, 3.5],
                               [6.33333333, 4.],
                               [6.33333333, 4.],
                               [20., 4.],
                               [12.66666667, 0.5],
                               [12.66666667, 1.],
                               [12.66666667, 1.],
                               [20., 1.],
                               [6.33333333, 1.25],
                               [6.33333333, 2.],
                               [6.33333333, 2.],
                               [20., 2.],
                               [0., 2.375],
                               [0., 3.5],
                               [0., 3.5],
                               [6.33333333, 3.5],
                               [6.33333333, 3.5],
                               [6.33333333, 3.],
                               [6.33333333, 3.],
                               [20., 3.],
                               [12.66666667, 0.5],
                               [12.66666667, 0.],
                               [12.66666667, 0.],
                               [20., 0.],
                               [6.33333333, 1.25],
                               [6.33333333, 0.5],
                               [6.33333333, 0.5],
                               [12.66666667, 0.5],
                               [0., 2.375],
                               [0., 1.25],
                               [0., 1.25],
                               [6.33333333, 1.25]])

        res = np.vstack([i._xy for i in lines])

        npt.assert_allclose(exp_coords, res)

        # Make sure that the metadata labels are set properly
        res = str(fig.get_axes()[2].get_xticklabels(minor=True)[0])
        self.assertEqual(res, "Text(0,0,'a')")

        res = str(fig.get_axes()[2].get_xticklabels(minor=True)[1])
        self.assertEqual(res, "Text(0,0,'b')")

        # Make sure that the highlight labels are set properly
        res = str(fig.get_axes()[1].get_xticklabels()[0])
        self.assertEqual(res, "Text(0,0,'y7')")

        res = str(fig.get_axes()[1].get_xticklabels()[1])
        self.assertEqual(res, "Text(0,0,'y8')")

        # Test to see if the highlights are ok
        res = fig.get_axes()[2].get_position()._points
        exp = np.array([[0.24, 0.1],
                        [0.808, 0.9]])
        npt.assert_allclose(res, exp)


if __name__ == "__main__":
    unittest.main()



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import abc
import pandas as pd
from skbio.stats.composition import ilr_inv
from gneiss._model import Model
from gneiss.balances import balance_basis


class RegressionModel(Model):
    def __init__(self, *args, **kwargs):
        """
        Summary object for storing regression results.

        A `RegressionResults` object stores information about the
        individual balances used in the regression, the coefficients,
        residuals. This object can be used to perform predictions.
        In addition, summary statistics such as the coefficient
        of determination for the overall fit can be calculated.

        Parameters
        ----------
        submodels : list of statsmodels objects
            List of statsmodels result objects.
        balances : pd.DataFrame
            A table of balances where samples are rows and
            balances are columns.  These balances were calculated
            using `tree`.
        """
        self._beta = None
        self._resid = None
        self._fitted = False
        super().__init__(*args, **kwargs)
        # there is only one design matrix for regression
        self.design_matrix = self.design_matrices

    def coefficients(self, tree=None):
        """ Returns coefficients from fit.

        Parameters
        ----------
        tree : skbio.TreeNode, optional
            The tree used to perform the ilr transformation.  If this
            is specified, then the prediction will be represented as
            proportions. Otherwise, if this is not specified, the prediction
            will be represented as balances. (default: None).

        Returns
        -------
        pd.DataFrame
            A table of coefficients where rows are covariates,
            and the columns are balances. If `tree` is specified, then
            the columns are proportions.
        """
        if not self._fitted:
            ValueError(('Model not fitted - coefficients not calculated.'
                        'See `fit()`'))
        coef = self._beta
        if tree is not None:
            basis, _ = balance_basis(tree)
            c = ilr_inv(coef.values, basis=basis)
            ids = [n.name for n in tree.tips()]
            return pd.DataFrame(c, columns=ids, index=coef.index)
        else:
            return coef

    def residuals(self, tree=None):
        """ Returns calculated residuals from fit.

        Parameters
        ----------
        X : pd.DataFrame, optional
            Input table of covariates.  If not specified, then the
            fitted values calculated from training the model will be
            returned.
        tree : skbio.TreeNode, optional
            The tree used to perform the ilr transformation.  If this
            is specified, then the prediction will be represented
            as proportions. Otherwise, if this is not specified,
            the prediction will be represented as balances. (default: None).

        Returns
        -------
        pd.DataFrame
            A table of residuals where rows are covariates,
            and the columns are balances. If `tree` is specified, then
            the columns are proportions.

        References
        ----------
        .. [1] Aitchison, J. "A concise guide to compositional data analysis,
           CDA work." Girona 24 (2003): 73-81.
        """
        if not self._fitted:
            ValueError(('Model not fitted - coefficients not calculated.'
                        'See `fit()`'))
        resid = self._resid
        if tree is not None:
            basis, _ = balance_basis(tree)
            proj_resid = ilr_inv(resid.values, basis=basis)
            ids = [n.name for n in tree.tips()]
            return pd.DataFrame(proj_resid,
                                columns=ids,
                                index=resid.index)
        else:
            return resid

    @abc.abstractmethod
    def predict(self, X=None, tree=None, **kwargs):
        """ Performs a prediction based on model.

        Parameters
        ----------
        X : pd.DataFrame, optional
            Input table of covariates, where columns are covariates, and
            rows are samples.  If not specified, then the fitted values
            calculated from training the model will be returned.
        tree : skbio.TreeNode, optional
            The tree used to perform the ilr transformation.  If this
            is specified, then the prediction will be represented
            as proportions. Otherwise, if this is not specified,
            the prediction will be represented as balances. (default: None).
        **kwargs : dict
            Other arguments to be passed into the model prediction.

        Returns
        -------
        pd.DataFrame
            A table of predicted values where rows are covariates,
            and the columns are balances. If `tree` is specified, then
            the columns are proportions.

        """
        if not self._fitted:
            ValueError(('Model not fitted - coefficients not calculated.'
                        'See `fit()`'))
        if X is None:
            X = self.design_matrices

        prediction = X.dot(self._beta)
        if tree is not None:
            basis, _ = balance_basis(tree)
            proj_prediction = ilr_inv(prediction.values, basis=basis)
            ids = [n.name for n in tree.tips()]
            return pd.DataFrame(proj_prediction,
                                columns=ids,
                                index=prediction.index)
        else:
            return prediction



"""
Clustering functions (:mod:`gneiss.cluster`)
============================================

.. currentmodule:: gneiss.cluster

This module contains functions to build hierarchical clusterings.


Functions
---------

.. autosummary::
   :toctree: generated/

       correlation_linkage
       gradient_linkage
       rank_linkage
       random_linkage

"""
# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
from ._pba import (correlation_linkage, gradient_linkage,
                   rank_linkage, random_linkage)


__all__ = ['correlation_linkage', 'gradient_linkage',
           'rank_linkage', 'random_linkage']



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import pandas as pd
from gneiss.plot._dendrogram import UnrootedDendrogram
from bokeh.models.glyphs import Circle, Segment
from bokeh.models import ColumnDataSource, DataRange1d, Plot
from bokeh.models import (HoverTool, BoxZoomTool, ResetTool,
                          WheelZoomTool, SaveTool, PanTool)


def radialplot(tree, node_color='node_color', node_size='node_size',
               node_alpha='node_alpha', edge_color='edge_color',
               edge_alpha='edge_alpha', edge_width='edge_width',
               hover_var='hover_var', figsize=(500, 500), **kwargs):
    """ Plots unrooted radial tree.

    Parameters
    ----------
    tree : instance of skbio.TreeNode
       Input tree for plotting.
    node_color : str
       Name of variable in `tree` to color nodes.
    node_size : str
       Name of variable in `tree` that specifies the radius of nodes.
    node_alpha : str
       Name of variable in `tree` to specify node transparency.
    edge_color : str
       Name of variable in `tree` to color edges.
    edge_alpha : str
       Name of variable in `tree` to specify edge transparency.
    edge_width : str
       Name of variable in `tree` to specify edge width.
    hover_var : str
       Name of variable in `tree` to display in the hover menu.
    figsize : tuple, int
       Size of resulting figure.  default: (500, 500)
    **kwargs: dict
       Plotting options to pass into bokeh.models.Plot

    Returns
    -------
    bokeh.models.Plot
       Interactive plotting instance.


    Notes
    -----
    This assumes that the tree is strictly bifurcating.

    See also
    --------
    bifurcate
    """
    # TODO: Add in example doc string

    # This entire function was motivated by
    # http://chuckpr.github.io/blog/trees2.html
    t = UnrootedDendrogram.from_tree(tree.copy())

    nodes = t.coords(figsize[0], figsize[1])

    # fill in all of the node attributes
    def _retreive(tree, x, default):
        return pd.Series({n.name: getattr(n, x, default)
                          for n in tree.levelorder()})

    # default node color to light grey
    nodes[node_color] = _retreive(t, node_color, default='#D3D3D3')
    nodes[node_size] = _retreive(t, node_size, default=1)
    nodes[node_alpha] = _retreive(t, node_alpha, default=1)
    nodes[hover_var] = _retreive(t, hover_var, default=None)

    edges = nodes[['child0', 'child1']]
    edges = edges.dropna(subset=['child0', 'child1'])
    edges = edges.unstack()
    edges = pd.DataFrame({'src_node': edges.index.get_level_values(1),
                          'dest_node': edges.values})
    edges['x0'] = [nodes.loc[n].x for n in edges.src_node]
    edges['x1'] = [nodes.loc[n].x for n in edges.dest_node]
    edges['y0'] = [nodes.loc[n].y for n in edges.src_node]
    edges['y1'] = [nodes.loc[n].y for n in edges.dest_node]
    ns = [n.name for n in t.levelorder(include_self=True)]
    attrs = pd.DataFrame(index=ns)

    # default edge color to black
    attrs[edge_color] = _retreive(t, edge_color, default='#000000')
    attrs[edge_width] = _retreive(t, edge_width, default=1)
    attrs[edge_alpha] = _retreive(t, edge_alpha, default=1)

    edges = pd.merge(edges, attrs, left_on='dest_node',
                     right_index=True, how='outer')
    edges = edges.dropna(subset=['src_node'])

    node_glyph = Circle(x="x", y="y",
                        radius=node_size,
                        fill_color=node_color,
                        fill_alpha=node_alpha)

    edge_glyph = Segment(x0="x0", y0="y0",
                         x1="x1", y1="y1",
                         line_color=edge_color,
                         line_alpha=edge_alpha,
                         line_width=edge_width)

    def df2ds(df):
        return ColumnDataSource(ColumnDataSource.from_df(df))

    ydr = DataRange1d(range_padding=0.05)
    xdr = DataRange1d(range_padding=0.05)

    plot = Plot(x_range=xdr, y_range=ydr, **kwargs)
    plot.add_glyph(df2ds(edges), edge_glyph)
    ns = plot.add_glyph(df2ds(nodes), node_glyph)

    tooltip = [
        ("Feature ID", "@index")
    ]
    if hover_var is not None:
        tooltip += [(hover_var, "@" + hover_var)]

    hover = HoverTool(renderers=[ns], tooltips=tooltip)
    plot.add_tools(hover, BoxZoomTool(), ResetTool(),
                   WheelZoomTool(), SaveTool(), PanTool())

    return plot



"""
Composition functions (:mod:`gneiss.composition`)
===============================================

.. currentmodule:: gneiss.composition

This module contains compositional functions

Functions
---------

.. autosummary::
   :toctree: generated/

   variation_matrix

"""
# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
from ._composition import ilr_transform
from ._variance import variation_matrix


__all__ = ["ilr_transform", "variation_matrix"]



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import unittest
import numpy as np
import pandas as pd
from gneiss.composition._composition import ilr_transform
from gneiss.cluster import gradient_linkage
import pandas.util.testing as pdt


class TestILRTransform(unittest.TestCase):

    def test_ilr(self):
        np.random.seed(0)
        table = pd.DataFrame([[1, 1, 2, 2],
                              [1, 2, 2, 1],
                              [2, 2, 1, 1]],
                             index=[1, 2, 3],
                             columns=['a', 'b', 'c', 'd'])
        table = table.reindex(columns=np.random.permutation(table.columns))
        ph = pd.Series([1, 2, 3], index=table.index)
        tree = gradient_linkage(table, ph)
        res_balances = ilr_transform(table, tree)
        exp_balances = pd.DataFrame(
            [[0.693147, -5.551115e-17, 2.775558e-17],
             [0.000000, -4.901291e-01, -4.901291e-01],
             [-0.693147, 5.551115e-17, -2.775558e-17]],
            columns=['y0', 'y1', 'y2'],
            index=[1, 2, 3])
        pdt.assert_frame_equal(res_balances, exp_balances)


if __name__ == '__main__':
    unittest.main()



"""
Regression functions (:mod:`gneiss.regression`)
===============================================

.. currentmodule:: gneiss.regression

This module contains functions that can convert proportions
to balances for regression analysis

Functions
---------

.. autosummary::
   :toctree: generated/

   ols
   mixedlm

Classes
-------
.. autosummary::
   :toctree: generated/

   OLSModel
   LMEModel

"""
# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
from ._ols import ols, OLSModel
from ._mixedlm import mixedlm, LMEModel


__all__ = ["ols", "OLSModel", "mixedlm", "LMEModel"]



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function
import unittest
import numpy as np
import pandas as pd
import numpy.testing as npt
from gneiss.balances import (balance_basis, _count_matrix,
                             _balance_basis, _attach_balances,
                             balanceplot)
from gneiss.layouts import default_layout
from skbio import TreeNode
from skbio.util import get_data_path


class TestPlot(unittest.TestCase):

    def test__attach_balances(self):
        tree = TreeNode.read([u"(a,b);"])
        balances = np.array([10])
        res_tree = _attach_balances(balances, tree)
        self.assertEqual(res_tree.weight, 10)

    def test__attach_balances_level_order(self):
        tree = TreeNode.read([u"((a,b)c,d)r;"])
        balances = np.array([10, -10])
        res_tree = _attach_balances(balances, tree)
        self.assertEqual(res_tree.weight, 10)
        self.assertEqual(res_tree.children[0].weight, -10)

    def test__attach_balances_bad_index(self):
        tree = TreeNode.read([u"((a,b)c,d)r;"])
        balances = np.array([10])
        with self.assertRaises(IndexError):
            _attach_balances(balances, tree)

    def test__attach_balances_series(self):
        tree = TreeNode.read([u"((a,b)c,d)r;"])
        balances = pd.Series([10, -10], index=['r', 'c'])
        res_tree = _attach_balances(balances, tree)
        self.assertEqual(res_tree.weight, 10)

    def test__attach_balances_series_bad(self):
        tree = TreeNode.read([u"((a,b)c,d)r;"])
        balances = pd.Series([10, -10])
        with self.assertRaises(KeyError):
            _attach_balances(balances, tree)

    def test_balanceplot(self):
        tree = TreeNode.read([u"((a,b)c,d)r;"])
        balances = np.array([10, -10])
        tr, ts = balanceplot(balances, tree)
        self.assertEquals(ts.mode, 'c')
        self.assertEquals(ts.layout_fn[0], default_layout)


class TestBalances(unittest.TestCase):

    def test_count_matrix_base_case(self):
        tree = u"(a,b);"
        t = TreeNode.read([tree])
        res, _ = _count_matrix(t)
        exp = {'k': 0, 'l': 1, 'r': 1, 't': 0, 'tips': 2}
        self.assertEqual(res[t], exp)

        exp = {'k': 0, 'l': 0, 'r': 0, 't': 0, 'tips': 1}
        self.assertEqual(res[t[0]], exp)

        exp = {'k': 0, 'l': 0, 'r': 0, 't': 0, 'tips': 1}
        self.assertEqual(res[t[1]], exp)

    def test_count_matrix_unbalanced(self):
        tree = u"((a,b)c, d);"
        t = TreeNode.read([tree])
        res, _ = _count_matrix(t)

        exp = {'k': 0, 'l': 2, 'r': 1, 't': 0, 'tips': 3}
        self.assertEqual(res[t], exp)
        exp = {'k': 1, 'l': 1, 'r': 1, 't': 0, 'tips': 2}
        self.assertEqual(res[t[0]], exp)

        exp = {'k': 0, 'l': 0, 'r': 0, 't': 0, 'tips': 1}
        self.assertEqual(res[t[1]], exp)
        self.assertEqual(res[t[0][0]], exp)
        self.assertEqual(res[t[0][1]], exp)

    def test_count_matrix_singleton_error(self):
        with self.assertRaises(ValueError):
            tree = u"(((a,b)c, d)root);"
            t = TreeNode.read([tree])
            _count_matrix(t)

    def test_count_matrix_trifurcating_error(self):
        with self.assertRaises(ValueError):
            tree = u"((a,b,e)c, d);"
            t = TreeNode.read([tree])
            _count_matrix(t)

    def test__balance_basis_base_case(self):
        tree = u"(a,b);"
        t = TreeNode.read([tree])

        exp_basis = np.array([[-np.sqrt(1. / 2), np.sqrt(1. / 2)]])
        exp_keys = [t]
        res_basis, res_keys = _balance_basis(t)

        npt.assert_allclose(exp_basis, res_basis)
        self.assertListEqual(exp_keys, res_keys)

    def test__balance_basis_unbalanced(self):
        tree = u"((a,b)c, d);"
        t = TreeNode.read([tree])

        exp_basis = np.array([[-np.sqrt(1. / 6), -np.sqrt(1. / 6),
                               np.sqrt(2. / 3)],
                              [-np.sqrt(1. / 2), np.sqrt(1. / 2), 0]
                              ])
        exp_keys = [t, t[0]]
        res_basis, res_keys = _balance_basis(t)

        npt.assert_allclose(exp_basis, res_basis)
        self.assertListEqual(exp_keys, res_keys)

    def test_balance_basis_base_case(self):
        tree = u"(a,b);"
        t = TreeNode.read([tree])
        exp_keys = [t]
        exp_basis = np.array([0.19557032, 0.80442968])
        res_basis, res_keys = balance_basis(t)

        npt.assert_allclose(exp_basis, res_basis)
        self.assertListEqual(exp_keys, res_keys)

    def test_balance_basis_unbalanced(self):
        tree = u"((a,b)c, d);"
        t = TreeNode.read([tree])
        exp_keys = [t, t[0]]
        exp_basis = np.array([[0.18507216, 0.18507216, 0.62985567],
                              [0.14002925, 0.57597535, 0.28399541]])

        res_basis, res_keys = balance_basis(t)

        npt.assert_allclose(exp_basis, res_basis)
        self.assertListEqual(exp_keys, list(res_keys))

    def test_balance_basis_large1(self):
        fname = get_data_path('large_tree.nwk',
                              subfolder='data')
        t = TreeNode.read(fname)
        # note that the basis is in reverse level order
        exp_basis = np.loadtxt(
            get_data_path('large_tree_basis.txt',
                          subfolder='data'))
        res_basis, res_keys = balance_basis(t)
        npt.assert_allclose(exp_basis[:, ::-1], res_basis)


if __name__ == "__main__":
    unittest.main()



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import os
import shutil
import unittest
import numpy as np
import pandas as pd
import pandas.util.testing as pdt
from skbio.stats.composition import ilr_inv
from skbio import TreeNode
from skbio.util import get_data_path
from gneiss.regression import ols
from gneiss.balances import balance_basis
import statsmodels.formula.api as smf


class TestOLS(unittest.TestCase):
    """ Tests OLS regression with refactored matrix multiplication. """
    def setUp(self):
        np.random.seed(0)
        b01, b11, b21 = 1, 2, -3
        b02, b12, b22 = 2, -1, 4
        n = 50
        x1 = np.linspace(0, 10, n)
        x2 = np.linspace(10, 15, n)
        e = np.random.normal(size=n) * 10
        y1 = b01 + b11 * x1 + b21 * x2 + e
        e = np.random.normal(size=n) * 10
        y2 = b02 + b12 * x1 + b22 * x2 + e
        Y = pd.DataFrame(np.vstack((y1, y2)).T,
                         columns=['y1', 'y2'])

        B = pd.DataFrame([[b01, b11, b21],
                          [b02, b12, b22]])

        X = pd.DataFrame(
            np.vstack((np.ones(n), x1, x2)).T,
            columns=['Intercept', 'x1', 'x2'])

        self.Y = Y
        self.B = B
        self.X = X
        self.r1_ = smf.OLS(endog=y1, exog=X).fit()
        self.r2_ = smf.OLS(endog=y2, exog=X).fit()
        self.tree = TreeNode.read(['(c, (b,a)y2)y1;'])

        self.results = "results"
        if not os.path.exists(self.results):
            os.mkdir(self.results)

    def tearDown(self):
        shutil.rmtree(self.results)

    def test_ols_immutable(self):
        # test to see if values in table get filtered out.
        # and that the original table doesn't change
        table = self.Y
        x = pd.DataFrame(self.X.values, columns=self.X.columns,
                         index=range(100, 100+len(self.X.index)))
        metadata = pd.concat((self.X, x))

        exp_metadata = metadata.copy()
        ols('x1 + x2', self.Y, self.X)
        self.assertEqual(str(table), str(self.Y))
        self.assertEqual(str(metadata), str(exp_metadata))

    def test_ols_missing_metadata(self):
        # test to see if values in table get filtered out.
        # and that the original table doesn't change
        table = self.Y
        y = pd.DataFrame(self.Y.values, columns=self.Y.columns,
                         index=range(100, 100+len(self.Y.index)))

        table = pd.concat((self.Y, y))
        ids = np.arange(100, 100+len(self.X.index))
        x = pd.DataFrame([[np.nan] * len(self.X.columns)] * len(ids),
                         columns=self.X.columns, index=ids)

        metadata = pd.concat((self.X, x))
        model = ols('x1 + x2', table, metadata)
        model.fit()

        # test prediction
        exp = pd.DataFrame({'y1': self.r1_.predict(),
                            'y2': self.r2_.predict()},
                           index=self.Y.index)
        res = model.predict()

        pdt.assert_frame_equal(res, exp)

    def test_ols_test(self):

        model = ols('x1 + x2', self.Y, self.X)
        model.fit()

        # test pvalues
        exp = pd.DataFrame({'y1': self.r1_.pvalues,
                            'y2': self.r2_.pvalues})
        pdt.assert_frame_equal(model.pvalues, exp)

        # test coefficients
        exp = pd.DataFrame({'y1': self.r1_.params,
                            'y2': self.r2_.params})
        res = model.coefficients()
        pdt.assert_frame_equal(res, exp)

        # test residuals
        exp = pd.DataFrame({'y1': self.r1_.resid,
                            'y2': self.r2_.resid},
                           index=self.Y.index)
        res = model.residuals()
        pdt.assert_frame_equal(res, exp)

        # test prediction
        exp = pd.DataFrame({'y1': self.r1_.predict(),
                            'y2': self.r2_.predict()},
                           index=self.Y.index)
        res = model.predict()
        pdt.assert_frame_equal(res, exp)

        # make a small prediction
        fx = pd.DataFrame(
            [[1, 1, 1],
             [1, 1, 2]],
            columns=['Intercept', 'x1', 'x2'],
            index=['f1', 'f2'])

        rp1 = self.r1_.predict([[1, 1, 1],
                                [1, 1, 2]])
        rp2 = self.r2_.predict([[1, 1, 1],
                                [1, 1, 2]])
        exp = pd.DataFrame({'y1': rp1,
                            'y2': rp2},
                           index=['f1', 'f2'])

        res = model.predict(X=fx)
        pdt.assert_frame_equal(res,  exp)

        # test r2
        self.assertAlmostEqual(model.r2, 0.21981627865598752)

    def test_ols_ilr_inv_test(self):

        model = ols('x1 + x2', self.Y, self.X)
        model.fit()
        basis, _ = balance_basis(self.tree)
        # test pvalues
        exp = pd.DataFrame({'y1': self.r1_.pvalues,
                            'y2': self.r2_.pvalues})
        pdt.assert_frame_equal(model.pvalues, exp)

        # test coefficients
        exp = pd.DataFrame({'y1': self.r1_.params,
                            'y2': self.r2_.params})

        exp = pd.DataFrame(ilr_inv(exp, basis),
                           columns=['c', 'b', 'a'],
                           index=self.X.columns)

        res = model.coefficients(tree=self.tree)
        pdt.assert_frame_equal(res, exp)

        # test residuals
        exp = pd.DataFrame({'y1': self.r1_.resid,
                            'y2': self.r2_.resid},
                           index=self.Y.index)
        exp = pd.DataFrame(ilr_inv(exp, basis),
                           index=self.Y.index,
                           columns=['c', 'b', 'a'])
        res = model.residuals(tree=self.tree)
        pdt.assert_frame_equal(res, exp)

        # test prediction
        exp = pd.DataFrame({'y1': self.r1_.predict(),
                            'y2': self.r2_.predict()},
                           index=self.Y.index)
        exp = pd.DataFrame(ilr_inv(exp, basis),
                           index=self.Y.index,
                           columns=['c', 'b', 'a'])
        res = model.predict(tree=self.tree)
        pdt.assert_frame_equal(res, exp)

    def test_tvalues(self):
        model = ols('x1 + x2', self.Y, self.X)
        model.fit()

        exp = pd.DataFrame({'y1': self.r1_.tvalues,
                            'y2': self.r2_.tvalues})
        pdt.assert_frame_equal(model.tvalues, exp)

    def test_mse(self):
        model = ols('x1 + x2', self.Y, self.X)
        model.fit()

        exp = pd.Series({'y1': self.r1_.mse_resid,
                         'y2': self.r2_.mse_resid})
        pdt.assert_series_equal(model.mse, exp)

    def test_ess(self):
        model = ols('x1 + x2', self.Y, self.X)
        model.fit()

        exp = pd.Series({'y1': self.r1_.ess,
                         'y2': self.r2_.ess})
        pdt.assert_series_equal(model.ess, exp)

    def test_loo(self):
        model = ols('x1 + x2', self.Y, self.X)
        model.fit()
        res = model.loo()
        exp = pd.read_csv(get_data_path('loo.csv'), index_col=0)
        pdt.assert_frame_equal(res, exp)

    def test_kfold(self):
        model = ols('x1 + x2', self.Y, self.X)
        model.fit()
        res = model.kfold(9)
        exp = pd.read_csv(get_data_path('kfold.csv'), index_col=0)
        pdt.assert_frame_equal(res, exp)

    def test_lovo(self):
        model = ols('x1 + x2', self.Y, self.X)
        model.fit()
        res = model.lovo()
        exp = pd.read_csv(get_data_path('lovo.csv'), index_col=0)
        pdt.assert_frame_equal(res, exp)


class TestOLSCV(unittest.TestCase):
    """ Tests OLS regression with refactored matrix multiplication. """
    def setUp(self):
        np.random.seed(0)
        b01, b11, b21 = 1, 2, -3
        b02, b12, b22 = 2, -1, 4
        n = 50
        x1 = np.linspace(0, 10, n)
        x2 = np.linspace(10, 15, n)**2
        e = np.random.normal(size=n) * 10
        y1 = b01 + b11 * x1 + b21 * x2 + e
        e = np.random.normal(size=n) * 10
        y2 = b02 + b12 * x1 + b22 * x2 + e
        Y = pd.DataFrame(np.vstack((y1, y2)).T,
                         columns=['y1', 'y2'])

        B = pd.DataFrame([[b01, b11, b21],
                          [b02, b12, b22]])

        X = pd.DataFrame(
            np.vstack((np.ones(n), x1, x2)).T,
            columns=['Intercept', 'x1', 'x2'])

        self.Y = Y
        self.B = B
        self.X = X
        self.r1_ = smf.OLS(endog=y1, exog=X).fit()
        self.r2_ = smf.OLS(endog=y2, exog=X).fit()
        self.tree = TreeNode.read(['(c, (b,a)y2)y1;'])

    def test_loo(self):
        model = ols('x1 + x2', self.Y, self.X)
        model.fit()
        res = model.loo()
        exp = pd.read_csv(get_data_path('loo2.csv'), index_col=0)
        pdt.assert_frame_equal(res, exp)

    def test_kfold(self):
        model = ols('x1 + x2', self.Y, self.X)
        model.fit()
        res = model.kfold(9)
        exp = pd.read_csv(get_data_path('kfold2.csv'), index_col=0)
        pdt.assert_frame_equal(res, exp)

    def test_lovo(self):
        model = ols('x1 + x2', self.Y, self.X)
        model.fit()
        res = model.lovo()
        exp = pd.read_csv(get_data_path('lovo2.csv'), index_col=0)
        pdt.assert_frame_equal(res, exp)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import re
import ast
import os

from setuptools import find_packages, setup
from setuptools.command.build_ext import build_ext as _build_ext


class build_ext(_build_ext):
    def finalize_options(self):
        _build_ext.finalize_options(self)
        # Prevent numpy from thinking it is still in its setup process:
        __builtins__.__NUMPY_SETUP__ = False
        import numpy
        self.include_dirs.append(numpy.get_include())


# Dealing with Cython
USE_CYTHON = os.environ.get('USE_CYTHON', False)
ext = '.pyx' if USE_CYTHON else '.c'

extensions = [
]

if USE_CYTHON:
    from Cython.Build import cythonize
    extensions = cythonize(extensions)

classes = """
    Development Status :: 4 - Beta
    License :: OSI Approved :: BSD License
    Topic :: Software Development :: Libraries
    Topic :: Scientific/Engineering
    Topic :: Scientific/Engineering :: Bio-Informatics
    Programming Language :: Python :: 3
    Programming Language :: Python :: 3 :: Only
    Operating System :: Unix
    Operating System :: POSIX
    Operating System :: MacOS :: MacOS X
"""
classifiers = [s.strip() for s in classes.split('\n') if s]

description = ('Compositional data analysis tools and visualizations')

with open('README.md') as f:
    long_description = f.read()


# version parsing from __init__ pulled from Flask's setup.py
# https://github.com/mitsuhiko/flask/blob/master/setup.py
_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('gneiss/__init__.py', 'rb') as f:
    hit = _version_re.search(f.read().decode('utf-8')).group(1)
    version = str(ast.literal_eval(hit))

setup(name='gneiss',
      version=version,
      license='BSD',
      description=description,
      long_description=long_description,
      author="gneiss development team",
      author_email="jamietmorton@gmail.com",
      maintainer="gneiss development team",
      maintainer_email="jamietmorton@gmail.com",
      packages=find_packages(),
      setup_requires=['numpy >= 1.9.2'],
      ext_modules=extensions,
      cmdclass={'build_ext': build_ext},
      install_requires=[
          'IPython >= 3.2.0',
          'matplotlib >= 1.4.3',
          'numpy >= 1.9.2',
          'pandas >= 0.18.0',
          'scipy >= 0.15.1',
          'nose >= 1.3.7',
          'scikit-bio==0.5.1',
          'statsmodels>=0.8.0',
          'seaborn'
      ],
      classifiers=classifiers,
      package_data={
          },
      )



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import pandas as pd
import statsmodels.formula.api as smf
from skbio import TreeNode
from gneiss._model import Model
import unittest
import os
import pandas.util.testing as pdt


# create some mock classes for testing
class submock_ok(Model):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def summary(self):
        print("OK!")

    def fit(self, **kwargs):
        pass


class submock_bad(Model):
    def __init__(self, **kwargs):
        super(Model, self, **kwargs)


class TestModel(unittest.TestCase):
    def setUp(self):
        self.pickle_fname = "test.pickle"
        self.data = pd.DataFrame([[1, 1, 1],
                                  [3, 2, 3],
                                  [4, 3, 2],
                                  [5, 4, 4],
                                  [2, 5, 3],
                                  [3, 6, 5],
                                  [4, 7, 4]],
                                 index=['s1', 's2', 's3', 's4',
                                        's5', 's6', 's7'],
                                 columns=['Y1', 'Y2', 'X'])

        self.model1 = smf.ols(formula="Y1 ~ X", data=self.data)
        self.model2 = smf.ols(formula="Y2 ~ X", data=self.data)

        self.basis = pd.DataFrame([[0.80442968, 0.19557032]],
                                  index=['a'],
                                  columns=['x', 'y'])
        self.tree = TreeNode.read(['(x, y)a;'])
        self.balances = pd.DataFrame({'a': [-1, 0, 1]})
        self.metadata = pd.DataFrame(
            [[1], [3], [2]],
            columns=['X'])

    def tearDown(self):
        if os.path.exists(self.pickle_fname):
            os.remove(self.pickle_fname)

    def test_init(self):
        res = submock_ok(Y=self.balances, Xs=self.metadata)

        # check balances
        pdt.assert_frame_equal(self.balances, res.response_matrix)


if __name__ == '__main__':
    unittest.main()



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import pandas as pd
from gneiss.plot._dendrogram import SquareDendrogram
from gneiss.util import match_tips, NUMERATOR, DENOMINATOR


def heatmap(table, tree, mdvar, highlights=None, cmap='viridis',
            linewidth=0.5, grid_col='w', grid_width=2, highlight_width=0.02,
            figsize=(5, 5), **kwargs):
    """ Creates heatmap plotting object

    Parameters
    ----------
    table : pd.DataFrame
        Contain sample/feature labels along with table of values.
        Rows correspond to samples, and columns correspond to features.
    tree: skbio.TreeNode
        Tree representing the feature hierarchy.
    highlights: pd.DataFrame or dict of tuple of str
        List of internal nodes in the tree to highlight.
        Each internal node must contain two colors, one for the left
        subtree and the other for the right subtree highlight.
        The first color will always correspond to the left subtree,
        and the second color will always correspond to the right subtree.
    cmap : str
        Specifies the matplotlib colormap for the heatmap (default='viridis')
    linewidth : int
        Width of dendrogram lines.
    mdvar: pd.Series
        Metadata values for samples.  The index must correspond to the
        index of `table`.
    highlight_width : int
        Width of highlights. (default=0.02)
    grid_col: str
        Color of vertical lines for highlighting sample metadata.
        (default='w')
    grid_width: int
        Width of vertical lines for highlighting sample metadata.
        (default=2)
    figsize: tuple of int
        Species (width, height) for figure. (default=(5, 5))
    **kwargs : dict
        Arguments to be passed into the heatmap plotting.

    Returns
    -------
    matplotlib.pyplot.figure
        Matplotlib figure object

    Note
    ----
    The highlights parameter assumes that the tree is bifurcating.
    """
    # match the tips
    dendrogram_width = 20

    table, tree = match_tips(table, tree)
    table = table.T
    # get edges from tree
    t = SquareDendrogram.from_tree(tree)
    t = _tree_coordinates(t)
    pts = t.coords(width=dendrogram_width, height=table.shape[0])
    edges = pts[['child0', 'child1']]
    edges = edges.dropna(subset=['child0', 'child1'])
    edges = edges.unstack()
    edges = pd.DataFrame({'src_node': edges.index.get_level_values(1),
                          'dest_node': edges.values})

    edge_list = []
    for i in edges.index:
        src = edges.loc[i, 'src_node']
        dest = edges.loc[i, 'dest_node']
        sx, sy = pts.loc[src].x, pts.loc[src].y
        dx, dy = pts.loc[dest].x, pts.loc[dest].y

        edge_list.append(
            {'x0': sx, 'y0': sy, 'x1': sx, 'y1': dy}
        )
        edge_list.append(
            {'x0': sx, 'y0': dy, 'x1': dx, 'y1': dy}
        )
    edge_list = pd.DataFrame(edge_list)

    # now plot the stuff
    fig = plt.figure(figsize=figsize, facecolor='white')
    plt.rcParams['axes.facecolor'] = 'white'
    xwidth = 0.2
    top_buffer = 0.1
    height = 0.8
    cbar_pad = 0.95

    # dendrogram axes on the left side
    [axm_x, axm_y, axm_w, axm_h] = [0, top_buffer, xwidth, height]

    # create a split for the highlights
    if highlights is not None:
        h = len(highlights)
    else:
        h = 0
    hwidth = highlight_width
    [axs_x, axs_y, axs_w, axs_h] = [xwidth, top_buffer, hwidth * h, height]

    # heatmap axes
    hstart = xwidth + (h * hwidth)
    [ax1_x, ax1_y, ax1_w, ax1_h] = [hstart, top_buffer,
                                    cbar_pad-hstart, height]

    # plot dendrogram
    ax_dendrogram = fig.add_axes([axm_x, axm_y, axm_w, axm_h],
                                 frame_on=True)
    _plot_dendrogram(ax_dendrogram, table, edge_list, linewidth=linewidth)

    # plot highlights for dendrogram
    if highlights is not None:
        ax_highlights = fig.add_axes([axs_x, axs_y, axs_w, axs_h],
                                     frame_on=True, sharey=ax_dendrogram)
        _plot_highlights_dendrogram(ax_highlights, table, t, highlights)

    # plot heatmap
    ax_heatmap = fig.add_axes([ax1_x, ax1_y, ax1_w, ax1_h], frame_on=True,
                              sharey=ax_dendrogram)
    cbar = _plot_heatmap(ax_heatmap, table, mdvar,
                         grid_col, grid_width, cmap, **kwargs)
    # plot the colorbar
    fig.colorbar(cbar)
    return fig


# TODO: Refactor and place in utils.  This can be also
# be used for the balance_basis calculations
def _tree_coordinates(t):
    """ Builds a matrix to link tree positions to matrix"""
    # first traverse the tree to count the children
    for n in t.postorder(include_self=True):
        if n.is_tip():
            n._n_tips = 1
        else:
            n._n_tips = sum(c._n_tips for c in n.children)

    for i, n in enumerate(t.levelorder(include_self=True)):
        if n.is_root():
            n._k = 0
            n._t = 0
        else:
            if n is n.parent.children[DENOMINATOR]:
                n._k = n.parent._k + n.parent._r
                n._t = n.parent._t
            else:
                n._k = n.parent._k
                n._t = n.parent._t + n.parent._l

        if n.is_tip():
            continue
        n._r = n.children[NUMERATOR]._n_tips
        n._l = n.children[DENOMINATOR]._n_tips
    return t


def _plot_highlights_dendrogram(ax_highlights, table, t, highlights):
    """ Plots highlights for subtrees in the dendrograms.

    Note that this assumes that the dendrograms are strictly bifurcating
    and the highlights only specify the children for a given subtree.
    """
    offset = 0.5

    num_h = len(highlights)
    hcoords = []
    for i, n in enumerate(highlights.index):
        node = t.find(n)

        k, l, r = node._k, node._l, node._r

        ax_highlights.add_patch(
            patches.Rectangle(
                (i/num_h, k-offset),  # x, y
                1/num_h,  # width
                r,  # height
                facecolor=highlights.iloc[i, 0]
            ))

        ax_highlights.add_patch(
            patches.Rectangle(
                (i/num_h, k+r-offset),  # x, y
                1/num_h,  # width
                l,  # height
                facecolor=highlights.iloc[i, 1]
            ))
        hcoords.append((i+offset)/num_h)
    ax_highlights.set_ylim([-offset, table.shape[0]-offset])
    ax_highlights.set_yticks([])
    ax_highlights.set_xticks(hcoords)
    ax_highlights.set_xticklabels(highlights.index, rotation=90)


def _plot_dendrogram(ax_dendrogram, table, edges, linewidth=1):
    """ Plots the actual dendrogram.

    Parameters
    ----------
    ax_dendrogram : matplotlib axes object
        Contains the matplotlib axes in which the dendrogram will be plotted.
    table : pd.DataFrame
        Contain sample/feature labels along with table of values.
        Rows correspond to samples, and columns correspond to features.
    edges : pd.DataFrame
        (x,y) coordinates for edges in the heatmap.
    """
    offset = 0.5
    for i in range(len(edges.index)):
        row = edges.iloc[i]
        ax_dendrogram.plot([row.x0, row.x1],
                           [row.y0-offset, row.y1-offset], '-k', lw=linewidth)
    ax_dendrogram.set_ylim([-offset, table.shape[0]-offset])
    ax_dendrogram.set_yticks([])
    ax_dendrogram.set_xticks([])
    ax_dendrogram.axis('off')


def _sort_table(table, mdvar):
    """ Sorts metadata category and aligns with table.

    Parameters
    ----------
    table : pd.DataFrame
        Contain sample/feature labels along with table of values.
        Rows correspond to samples, and columns correspond to features.
    mdvar : pd.Series
        Metadata values for samples.  The index must correspond to the
        index of `table`.

    Returns
    -------
    pd.DataFrame
        Aligned feature table.
    pd.Series
        Aligned metadata.
    """
    mdvar = mdvar.sort_values()
    table = table.reindex(columns=mdvar.index)
    return table, mdvar


def _plot_heatmap(ax_heatmap, table, mdvar, grid_col,
                  grid_width, cmap, **kwargs):
    """ Sorts metadata category and aligns with table.

    Parameters
    ----------
    ax_heatmap : matplotlib axes object
        Contains the matplotlib axes in which the heatmap will be plotted.
    table : pd.DataFrame
        Contain sample/feature labels along with table of values.
        Rows correspond to samples, and columns correspond to features.
    mdvar : pd.Series
        Metadata values for samples.  The index must correspond to the
        index of `table`.
    grid_col: str
        Color of vertical lines for highlighting sample metadata.
        (default='w')
    grid_width: int
        Width of vertical lines for highlighting sample metadata.
        (default=2)
    **kwargs: dict
        Arguments to be passed into the heatmap function.

    Returns
    -------
    colorbar_ax : matplotlib axes object
        The colorbar axis.
    """
    # TODO add explicit test for this, since matplotlib orientation
    # is from top to down (i.e. is backwards)
    table, mdvar = _sort_table(table, mdvar)
    table = table.iloc[::-1, :]
    colorbar_ax = ax_heatmap.imshow(table, aspect='auto',
                                    interpolation='nearest',
                                    cmap=cmap, **kwargs)
    ax_heatmap.set_ylim([0, table.shape[0]])
    vcounts = mdvar.value_counts()

    # ensure that the ordering is the same
    vcounts = vcounts.sort_index()

    ticks = vcounts.sort_index().cumsum()
    midpoints = ticks - (ticks - np.array([0] + list(ticks.values[:-1]))) / 2.0
    ax_heatmap.set_xticks(ticks.values-0.5, minor=False)
    ax_heatmap.set_xticklabels([], minor=False)

    ax_heatmap.xaxis.grid(True, which='major', color=grid_col,
                          linestyle='-', linewidth=grid_width)

    ax_heatmap.set_xticks(midpoints-0.5, minor=True)
    ax_heatmap.set_xticklabels(vcounts.index, minor=True)
    ax_heatmap.set_xlabel(mdvar.name)
    return colorbar_ax



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import abc
import numpy as np


class Model(metaclass=abc.ABCMeta):

    def __init__(self, Y, Xs):
        """
        Abstract container for balance models.

        Parameters
        ----------
        Y : pd.DataFrame
            Response matrix.  This is the matrix being predicted.
            Also known as the dependent variable in univariate analysis.
        Xs : iterable of pd.DataFrame
            Design matrices.  Also known as the independent variable
            in univariate analysis. Note that this allows for multiple
            design matrices to be inputted to enable multiple data block
            analysis.
        """
        self.response_matrix = Y
        self.design_matrices = Xs

    @abc.abstractmethod
    def fit(self, **kwargs):
        pass

    @abc.abstractmethod
    def summary(self):
        """ Print summary results """
        pass

    def percent_explained(self):
        """ Proportion explained by each principal balance."""
        # Using sum of squares error calculation (df=1)
        # instead of population variance (df=0).
        axis_vars = np.var(self.response_matrix, ddof=1, axis=0)
        return axis_vars / axis_vars.sum()



# ----------------------------------------------------------------------------
# Copyright (c) 2016--, gneiss development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import unittest
from gneiss.plot import balance_boxplot, balance_barplots, proportion_plot
import numpy as np
import pandas as pd
import numpy.testing as npt
import matplotlib.pyplot as plt
from skbio import TreeNode


class TestBoxplot(unittest.TestCase):
    def setUp(self):
        self.df = pd.DataFrame({
            'y': [-2, -2.2, -1.8, -1.5, -1, 1, 1.5, 2, 2.2, 1.8],
            'group': ['a', 'a', 'a', 'a', 'a', 'b', 'b', 'b', 'b', 'b'],
            'hue': ['0', '1', '0', '1', '0', '1', '0', '1', '0', '1']}
        )
        self.tree = TreeNode.read(['((c, d)z, (b,a)x)y;'])
        self.feature_df = pd.DataFrame(
            {
                'type': ['tomato', 'carrots', 'apple', 'bacon'],
                'food': ['vegatable', 'vegatable', 'fruit', 'meat'],
                'seed': ['yes', 'no', 'yes', 'no']
            },
            index=["a", "b", "c", "d"]
        )

    def test_basic_boxplot(self):
        a = balance_boxplot('y', y='group', data=self.df)
        res = np.vstack([i._xy for i in a.get_lines()])
        exp = np.array([[-2., 0.],
                        [-2.2, 0.],
                        [-1.5, 0.],
                        [-1., 0.],
                        [-2.2, -0.2],
                        [-2.2, 0.2],
                        [-1., -0.2],
                        [-1., 0.2],
                        [-1.8, -0.4],
                        [-1.8, 0.4],
                        [1.5, 1.],
                        [1., 1.],
                        [2., 1.],
                        [2.2, 1.],
                        [1., 0.8],
                        [1., 1.2],
                        [2.2, 0.8],
                        [2.2, 1.2],
                        [1.8, 0.6],
                        [1.8, 1.4]])
        npt.assert_allclose(res, exp)

    def test_basic_hue_boxplot(self):
        a = balance_boxplot('y', y='group', hue='hue', data=self.df)
        res = np.vstack([i._xy for i in a.get_lines()])
        exp = np.array([[-1.9, -0.2],
                        [-2., -0.2],
                        [-1.4, -0.2],
                        [-1., -0.2],
                        [-2., -0.298],
                        [-2., -0.102],
                        [-1., -0.298],
                        [-1., -0.102],
                        [-1.8, -0.396],
                        [-1.8, -0.004],
                        [-2.025, 0.2],
                        [-2.2, 0.2],
                        [-1.675, 0.2],
                        [-1.5, 0.2],
                        [-2.2, 0.102],
                        [-2.2, 0.298],
                        [-1.5, 0.102],
                        [-1.5, 0.298],
                        [-1.85, 0.004],
                        [-1.85, 0.396],
                        [1.675, 0.8],
                        [1.5, 0.8],
                        [2.025, 0.8],
                        [2.2, 0.8],
                        [1.5, 0.702],
                        [1.5, 0.898],
                        [2.2, 0.702],
                        [2.2, 0.898],
                        [1.85, 0.604],
                        [1.85, 0.996],
                        [1.4, 1.2],
                        [1., 1.2],
                        [1.9, 1.2],
                        [2., 1.2],
                        [1., 1.102],
                        [1., 1.298],
                        [2., 1.102],
                        [2., 1.298],
                        [1.8, 1.004],
                        [1.8, 1.396]])
        npt.assert_allclose(res, exp)

    def test_basic_barplot(self):
        ax_denom, ax_num = balance_barplots(self.tree, 'y', header='food',
                                            feature_metadata=self.feature_df)


class TestProportionPlot(unittest.TestCase):
    def setUp(self):
        self.table = pd.DataFrame({
            'A': [1, 1.2, 1.1, 2.1, 2.2, 2],
            'B': [9.9, 10, 10.1, 2, 2.4, 2.1],
            'C': [5, 3, 1, 2, 2, 3],
            'D': [5, 5, 5, 5, 5, 5],
        }, index=['S1', 'S2', 'S3', 'S4', 'S5', 'S6'])

        self.feature_metadata = pd.DataFrame({
            'A': ['k__foo', 'p__bar', 'c__', 'o__', 'f__', 'g__', 's__'],
            'B': ['k__foo', 'p__bar', 'c__', 'o__', 'f__', 'g__', 's__'],
            'C': ['k__poo', 'p__tar', 'c__', 'o__', 'f__', 'g__', 's__'],
            'D': ['k__poo', 'p__far', 'c__', 'o__', 'f__', 'g__', 's__']
        }, index=['kingdom', 'phylum', 'class', 'order',
                  'family', 'genus', 'species']).T

        self.metadata = pd.DataFrame({
            'groups': ['X', 'X', 'X', 'Y', 'Y', 'Y'],
            'dry': [1, 2, 3, 4, 5, 6]
        }, index=['S1', 'S2', 'S3', 'S4', 'S5', 'S6'])

    def test_proportion_plot(self):
        np.random.seed(0)
        num_features = ['A', 'B']
        denom_features = ['C', 'D']
        ax1, ax2 = proportion_plot(self.table, self.metadata,
                                   'groups', 'X', 'Y',
                                   num_features, denom_features,
                                   self.feature_metadata,
                                   label_col='phylum')
        res = np.vstack([l.get_xydata() for l in ax1.get_lines()])
        exp = np.array([0., 0., 1., 1., 2., 2., 3., 3.])

        npt.assert_allclose(res[:, 1], exp, verbose=True)

        res = np.vstack([l.get_xydata() for l in ax2.get_lines()])
        exp = np.array([0., 0., 1., 1., 2., 2., 3., 3.])

        npt.assert_allclose(res[:, 1], exp, verbose=True)

        res = [l._text for l in ax2.get_yticklabels()]
        exp = ['p__bar', 'p__bar', 'p__tar', 'p__far']
        self.assertListEqual(res, exp)

    def test_proportion_plot_order(self):
        self.maxDiff = None
        np.random.seed(0)
        # tests for different ordering
        num_features = ['A', 'B']
        denom_features = ['D', 'C']
        ax1, ax2 = proportion_plot(self.table, self.metadata,
                                   'groups', 'X', 'Y',
                                   num_features, denom_features,
                                   self.feature_metadata,
                                   label_col='phylum')
        res = np.vstack([l.get_xydata() for l in ax1.get_lines()])
        exp = np.array([0., 0., 1., 1., 2., 2., 3., 3.])

        npt.assert_allclose(res[:, 1], exp, atol=1e-2, rtol=1e-2, verbose=True)

        res = np.vstack([l.get_xydata() for l in ax2.get_lines()])
        exp = np.array([0., 0., 1., 1., 2., 2., 3., 3.])

        npt.assert_allclose(res[:, 1], exp, atol=1e-2, rtol=1e-2, verbose=True)

        res = [l._text for l in ax2.get_yticklabels()]
        exp = ['p__bar', 'p__bar', 'p__far', 'p__tar']
        self.assertListEqual(res, exp)

    def test_proportion_plot_order_figure(self):
        self.maxDiff = None
        np.random.seed(0)
        # tests for different ordering
        fig, axes = plt.subplots(1, 2)

        num_features = ['A', 'B']
        denom_features = ['D', 'C']
        ax1, ax2 = proportion_plot(self.table, self.metadata,
                                   'groups', 'X', 'Y',
                                   num_features, denom_features,
                                   self.feature_metadata,
                                   label_col='phylum', axes=axes)
        res = np.vstack([l.get_xydata() for l in ax1.get_lines()])
        exp = np.array([0., 0., 1., 1., 2., 2., 3., 3.])

        npt.assert_allclose(res[:, 1], exp, atol=1e-2, rtol=1e-2, verbose=True)

        res = np.vstack([l.get_xydata() for l in ax2.get_lines()])
        exp = np.array([0., 0., 1., 1., 2., 2., 3., 3.])

        npt.assert_allclose(res[:, 1], exp, atol=1e-2, rtol=1e-2, verbose=True)

        res = [l._text for l in ax2.get_yticklabels()]
        exp = ['p__bar', 'p__bar', 'p__far', 'p__tar']
        self.assertListEqual(res, exp)

    def test_proportion_plot_original_labels(self):
        # tests for different ordering
        fig, axes = plt.subplots(1, 2)

        num_features = ['A', 'B']
        denom_features = ['D', 'C']
        ax1, ax2 = proportion_plot(self.table, self.metadata,
                                   'groups', 'X', 'Y',
                                   num_features, denom_features,
                                   axes=axes)

        res = [l._text for l in ax2.get_yticklabels()]
        exp = ['A', 'B', 'D', 'C']
        self.assertListEqual(res, exp)


if __name__ == '__main__':
    unittest.main()
